
.icon-circles:before { content: '\e800'; } /* '' */
.icon-home:before { content: '\e801'; } /* '' */
.icon-help-circle:before { content: '\e802'; } /* '' */
.icon-book:before { content: '\e803'; } /* '' */
.icon-github-circled:before { content: '\e804'; } /* '' */
.icon-cancel-circled:before { content: '\e805'; } /* '' */
.icon-dot-circled:before { content: '\e806'; } /* '' */
.icon-circle-empty:before { content: '\e807'; } /* '' */
.icon-check-off:before { content: '\e808'; } /* '' */
.icon-check-on:before { content: '\e809'; } /* '' */
.icon-ok-circled:before { content: '\e80a'; } /* '' */
.icon-angle-circled-right:before { content: '\e80b'; } /* '' */
.icon-pencil:before { content: '\e80c'; } /* '' */
.icon-doc:before { content: '\e80d'; } /* '' */
.icon-doc-inv:before { content: '\e80e'; } /* '' */
.icon-hamburger:before { content: '\e80f'; } /* '' */
.icon-cog:before { content: '\e810'; } /* '' */
.icon-left:before { content: '\e811'; } /* '' */
.icon-menu-outline:before { content: '\e812'; } /* '' */
.icon-plus-circle:before { content: '\e813'; } /* '' */
.icon-down-circle:before { content: '\e814'; } /* '' */
.icon-up-circle:before { content: '\e815'; } /* '' */
.icon-angle-circled-left:before { content: '\e816'; } /* '' */
.icon-trash:before { content: '\e817'; } /* '' */
.icon-chart-line:before { content: '\e818'; } /* '' */
.icon-download:before { content: '\e819'; } /* '' */
.icon-user:before { content: '\e81a'; } /* '' */
.icon-menu:before { content: '\e81b'; } /* '' */
.icon-sidebar:before { content: '\e81c'; } /* '' */
.icon-sidebar-open:before { content: '\e81d'; } /* '' */
.icon-lightbulb:before { content: '\e81e'; } /* '' */
.icon-mail:before { content: '\e81f'; } /* '' */
.icon-key:before { content: '\e820'; } /* '' */
.icon-logout:before { content: '\e821'; } /* '' */
.icon-spin6:before { content: '\e822'; } /* '' */
.icon-flow-branch:before { content: '\e823'; } /* '' */
.icon-clipboard:before { content: '\e824'; } /* '' */
.icon-oira-tool:before { content: '\e825'; } /* '' */
.icon-oira-risk-assessment:before { content: '\e826'; } /* '' */
.icon-publish-old:before { content: '\e827'; } /* '' */
.icon-share:before { content: '\e828'; } /* '' */
.icon-globe:before { content: '\e829'; } /* '' */
.icon-right-open:before { content: '\e82a'; } /* '' */
.icon-search:before { content: '\e82b'; } /* '' */
.icon-info-circle:before { content: '\e82c'; } /* '' */
.icon-published:before { content: '\e82d'; } /* '' */
.icon-unpublished:before { content: '\e82e'; } /* '' */
.icon-publish:before { content: '\e82f'; } /* '' */
.icon-menu-bars:before { content: '\e830'; } /* '' */
.icon-cancel-1:before { content: '\e831'; } /* '' */
.icon-bank:before { content: '\e84c'; } /* '' */
.icon-ellipsis:before { content: '\e865'; } /* '' */
.icon-cancel:before { content: '\e899'; } /* '' */
.icon-arrows-cw:before { content: '\e89c'; } /* '' */
.icon-twitter:before { content: '\f099'; } /* '' */
.icon-menu-1:before { content: '\f0c9'; } /* '' */
.icon-ellipsis-vertical:before { content: '\f142'; } /* '' */
.icon-tree:before { content: '\f1bb'; } /* '' */
.icon-facebook:before { content: '\f230'; } /* '' */
.icon-clone:before { content: '\f24d'; } /* '' */